# -*- coding: mbcs -*-
#script for running Abaqus simulation
from part import *
from material import *
from section import *
from optimization import *
from assembly import *
from step import *
from interaction import *
from load import *
from mesh import *
from job import *
from sketch import *
from visualization import *
from connectorBehavior import *
# for prettyPrint:
from textRepr import *

ModelName='Model-10'
PartName='Testspec'
OdbName='testspecimen'

#Load the material parameters  C1, ga1, C2, ga2, sigma0
execfile('material.py')


# three symmetry planes
TestSectionLength=80.0/2
TestSectionWidth=10.0/2
GripLength=20.0
GripWidth=24.0/2
Radius=10.0
Thickness=1.81/2
GageLength=50.0/2

GripDisplacement=3
Backstresses=2
MeshSize=0.5
ElementsThroughThickness=4
FirstOrderMesh=1

try: 
	Model=mdb.models[ModelName]
except: 
	mdb.Model(modelType=STANDARD_EXPLICIT, name=ModelName)
	
Model=mdb.models[ModelName]		

Model.ConstrainedSketch(name='__profile__', sheetSize=200.0)
Profile=Model.sketches['__profile__']
Profile.Line(point1=(0.0, 0.0), point2=(
    5.0, 0.0))
Profile.HorizontalConstraint(
    addUndoState=False, entity=
    Profile.geometry[2])
Profile.Line(point1=(5.0, 0.0), point2=(
    5.0, 10.0))
Profile.VerticalConstraint(addUndoState=
    False, entity=Profile.geometry[3])
Profile.PerpendicularConstraint(
    addUndoState=False, entity1=
    Profile.geometry[2], entity2=
    Profile.geometry[3])
Profile.Line(point1=(7.5, 10.0), point2=(
    7.5, 20.0))
Profile.VerticalConstraint(addUndoState=
    False, entity=Profile.geometry[4])
Profile.Line(point1=(7.5, 20.0), point2=(
    0.0, 20.0))
Profile.HorizontalConstraint(
    addUndoState=False, entity=
    Profile.geometry[5])
Profile.PerpendicularConstraint(
    addUndoState=False, entity1=
    Profile.geometry[4], entity2=
    Profile.geometry[5])
Profile.Line(point1=(0.0, 20.0), point2=(
    0.0, 0.0))
Profile.VerticalConstraint(addUndoState=
    False, entity=Profile.geometry[6])
Profile.PerpendicularConstraint(
    addUndoState=False, entity1=
    Profile.geometry[5], entity2=
    Profile.geometry[6])
Profile.ArcByCenterEnds(center=(10.0, 
    10.0), direction=CLOCKWISE, point1=(5.0, 10.0), point2=(7.5, 
    16.6015110015869))
Profile.CoincidentConstraint(
    addUndoState=False, entity1=
    Profile.vertices[6], entity2=
    Profile.geometry[4])
Profile.autoTrimCurve(curve1=
    Profile.geometry[4], point1=(
    7.32020092010498, 11.3181343078613))
Profile.TangentConstraint(entity1=
    Profile.geometry[3], entity2=
    Profile.geometry[7])
Profile.VerticalConstraint(entity=
    Profile.geometry[8])
Profile.PerpendicularConstraint(entity1=
    Profile.geometry[2], entity2=
    Profile.geometry[6])
Profile.PerpendicularConstraint(entity1=
    Profile.geometry[5], entity2=
    Profile.geometry[8])
Profile.RadialDimension(curve=
    Profile.geometry[7], radius=Radius, 
    textPoint=(11.4393606185913, 8.73565864562988))
Profile.FixedConstraint(entity=
    Profile.vertices[0])
Profile.ObliqueDimension(textPoint=(
    9.16459941864014, 3.8166561126709), value=8, vertex1=
    Profile.vertices[1], vertex2=
    Profile.vertices[2])
Profile.ObliqueDimension(textPoint=(
    10.1482801437378, 17.8358135223389), value=2, vertex1=
    Profile.vertices[6], vertex2=
    Profile.vertices[4])
Profile.ObliqueDimension(textPoint=(
    3.33308601379395, -0.604379653930664), value=5.0, vertex1=
    Profile.vertices[0], vertex2=
    Profile.vertices[1])
Profile.ObliqueDimension(textPoint=(
    3.40046787261963, 20.3542308807373), value=7.5, vertex1=
    Profile.vertices[4], vertex2=
    Profile.vertices[5])
Model.Part(dimensionality=THREE_D, name=PartName, type=
    DEFORMABLE_BODY)
Part=Model.parts[PartName]    
Part.BaseSolidExtrude(depth=Thickness, sketch=
    Profile)
del Profile


# material parameters
Model.Material(name='Material-1')
Model.materials['Material-1'].Elastic(table=((210000.0, 0.3), 
    ))
if (Backstresses==1):
	Model.materials['Material-1'].Plastic(dataType=PARAMETERS, 
		hardening=COMBINED, table=((300.0, 100000.0, 1000.0), )) #sigma0, C, gamma
if (Backstresses==2):
	Model.materials['Material-1'].Plastic(dataType=PARAMETERS, 
		hardening=COMBINED, numBackstresses=2, table=((320.0, C1, ga1, C2, ga2), ))

		
Model.HomogeneousSolidSection(material='Material-1', name=
    'Section-1', thickness=None)
Part.Set(cells=
    Part.cells.getSequenceFromMask(('[#1 ]', 
    ), ), name='Whole')
Part.SectionAssignment(offset=0.0, 
    offsetField='', offsetType=MIDDLE_SURFACE, region=
    Part.sets['Whole'], sectionName=
    'Section-1', thicknessAssignment=FROM_SECTION)

# make a step    
Model.rootAssembly.DatumCsysByDefault(CARTESIAN)
Model.rootAssembly.Instance(dependent=ON, name='Testspec-1', 
    part=Part)
Instance=Model.rootAssembly.instances['Testspec-1']    
Model.StaticStep(initialInc=0.2, name='Step-1', nlgeom=ON, 
    previous='Initial')    

# strain gauge
Model.parts['Testspec'].DatumPlaneByPrincipalPlane(offset=5.0, 
    principalPlane=XZPLANE)
Part.PartitionCellByDatumPlane(cells=
    Part.cells.getSequenceFromMask(('[#1 ]', 
    ), ), datumPlane=Part.datums[3])
    
# boundary conditions
Model.rootAssembly.Set(faces=
    Instance.faces.getSequenceFromMask(
    ('[#20 ]', ), ), name='grip')
Model.DisplacementBC(amplitude=UNSET, createStepName='Step-1', 
    distributionType=UNIFORM, fieldName='', fixed=OFF, localCsys=None, name=
    'gripdisp', region=Model.rootAssembly.sets['grip'], u1=
    UNSET, u2=GripDisplacement, u3=UNSET, ur1=UNSET, ur2=UNSET, ur3=UNSET)
Model.rootAssembly.Set(faces=
    Instance.faces.getSequenceFromMask(
    ('[#200 ]', ), ), name='ysym')
Model.YsymmBC(createStepName='Step-1', localCsys=None, name=
    'ysym', region=Model.rootAssembly.sets['ysym'])
Model.rootAssembly.Set(faces=
    Instance.faces.getSequenceFromMask(
    ('[#410 ]', ), ), name='xsym')
Model.XsymmBC(createStepName='Step-1', localCsys=None, name=
    'xsym', region=Model.rootAssembly.sets['xsym'])
Model.rootAssembly.Set(faces=
    Instance.faces.getSequenceFromMask(
    ('[#1008 ]', ), ), name='zsym')
Model.ZsymmBC(createStepName='Step-1', localCsys=None, name=
    'zsym', region=Model.rootAssembly.sets['zsym'])
    
#mesh seeds
Part.setMeshControls(algorithm=MEDIAL_AXIS, 
    regions=Part.cells.getSequenceFromMask((
    '[#2 ]', ), ))
Part.seedPart(minSizeFactor=0.1, size=MeshSize)
Part.seedEdgeByNumber(constraint=FINER, 
    edges=Part.edges.getSequenceFromMask((
    '[#2 ]', ), ), number=ElementsThroughThickness)
Part.setElementType(elemTypes=(ElemType(
    elemCode=C3D20R, elemLibrary=STANDARD), ElemType(elemCode=C3D15, 
    elemLibrary=STANDARD), ElemType(elemCode=C3D10, elemLibrary=STANDARD)), 
    regions=(Part.cells.getSequenceFromMask(
    ('[#3 ]', ), ), ))


# strain history output at strain gauge attachment points
Model.rootAssembly.Set(name='StrainOut', vertices=
    Instance.vertices.getSequenceFromMask(
    ('[#f ]', ), ))
Model.HistoryOutputRequest(createStepName='Step-1', name=
    'H-Output-2', rebar=EXCLUDE, region=
    Model.rootAssembly.sets['StrainOut'], sectionPoints=DEFAULT
    , variables=('U2', ))    

# CHANGE: single node traction to get force sum    
Model.rootAssembly.ReferencePoint(point=
    Model.rootAssembly.instances['Testspec-1'].vertices[8])
Model.rootAssembly.Set(name='m_Set-6', referencePoints=(
    Model.rootAssembly.referencePoints[9], ))
Model.MultipointConstraint(controlPoint=
    Model.rootAssembly.sets['m_Set-6'], csys=None, mpcType=
    TIE_MPC, name='Constraint-1', surface=
    Model.rootAssembly.sets['grip'], userMode=DOF_MODE_MPC, 
    userType=0)
Model.boundaryConditions['gripdisp'].setValues(region=
    Model.rootAssembly.sets['m_Set-6'], u1=0.0, u3=0.0)

# history output for traction node    
Model.HistoryOutputRequest(createStepName='Step-1', name=
    'H-Output-3', rebar=EXCLUDE, region=
    Model.rootAssembly.sets['m_Set-6'], sectionPoints=DEFAULT, 
    variables=('RF2', ))    

if (FirstOrderMesh):
	Part.setElementType(elemTypes=(ElemType(
		elemCode=C3D8R, elemLibrary=STANDARD, secondOrderAccuracy=OFF, 
		kinematicSplit=AVERAGE_STRAIN, hourglassControl=DEFAULT, 
		distortionControl=DEFAULT), ElemType(elemCode=C3D6, elemLibrary=STANDARD), 
		ElemType(elemCode=C3D4, elemLibrary=STANDARD)), regions=(
		Part.cells.getSequenceFromMask(('[#3 ]', 
		), ), ))
	mdb.models['Model-1'].rootAssembly.regenerate()

#CHANGE: finer steps
mdb.models['Model-10'].steps['Step-1'].setValues(initialInc=0.001, maxInc=0.02, 
    maxNumInc=5000)	

Part.seedEdgeByNumber(constraint=FINER, 
    edges=Part.edges.getSequenceFromMask((
    '[#10000 ]', ), ), number=int(TestSectionWidth/MeshSize))
    


# final dimension
Model.ConstrainedSketch(name='__edit__', objectToCopy=
    Part.features['Solid extrude-1'].sketch)
Edit=Model.sketches['__edit__']   
Edit.dimensions[3].setValues(value=TestSectionWidth)
Edit.dimensions[1].setValues(value=TestSectionLength)
Edit.dimensions[0].setValues(value=Radius)
Edit.dimensions[2].setValues(value=GripLength)
Edit.dimensions[4].setValues(value=GripWidth)
Part.features['Solid extrude-1'].setValues(
    sketch=Edit)
del Edit
Part.regenerate()
Part.features['Datum plane-1'].setValues(
    offset=GageLength)
Part.regenerate()    


# mesh
Part.generateMesh()

Part.editNode(nodes=
    Part.nodes.getSequenceFromMask(mask=(
    '[#4 ]', ), ), offset1=-0.05)
    
Part.editNode(nodes=
    Part.nodes.getSequenceFromMask(mask=(
    '[#0:2 #10 ]', ), ), offset1=-0.05)

    

# create job
mdb.Job(atTime=None, contactPrint=OFF, description='', echoPrint=OFF, 
    explicitPrecision=SINGLE, getMemoryFromAnalysis=True, historyPrint=OFF, 
    memory=90, memoryUnits=PERCENTAGE, model=ModelName, modelPrint=OFF, 
    multiprocessingMode=DEFAULT, name=OdbName, nodalOutputPrecision=SINGLE, 
    numCpus=1, numGPUs=0, queue=None, resultsFormat=ODB, scratch='', type=
    ANALYSIS, userSubroutine='', waitHours=0, waitMinutes=0)
    
# run job    
mdb.jobs[OdbName].submit(consistencyChecking=OFF)    